package com.example.SparkExceptionExample;

import com.example.SparkExceptionExample.model.Promotion;
import com.fasterxml.jackson.databind.ObjectMapper;
import io.confluent.kafka.schemaregistry.client.rest.exceptions.RestClientException;
import lombok.extern.slf4j.Slf4j;
import org.apache.spark.api.java.JavaRDD;
import org.apache.spark.sql.*;
import org.apache.spark.sql.types.DataTypes;
import org.apache.spark.sql.types.StructField;
import org.apache.spark.sql.types.StructType;
import org.jooq.lambda.Unchecked;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.List;

@Slf4j
public class SparkExample {

    static SparkSession sparkSession;
    static StructType schema;
    final static String jsonPath = "/Users/nisum/Desktop/EPE/sparkstructtype/src/main/resources/promotions.json";
    static ObjectMapper mapper = new ObjectMapper();
    public static void main(String[] args) throws AnalysisException, IOException, RestClientException {
        sparkSession = SparkSession.builder().master("local").appName("Spark StructType").getOrCreate();
        schema = new StructType(new StructField[]{
                DataTypes.createStructField("promotionId", DataTypes.IntegerType, true),
                DataTypes.createStructField("promotionType", DataTypes.StringType, true),
                DataTypes.createStructField("discount", DataTypes.IntegerType, true),
        });
        String jsonString = new String(Files.readAllBytes(Paths.get("/Users/nisum/Desktop/EPE/sparkstructtype/src/main/resources/schemas/promotion.json")));
        StructType structType = (StructType) StructType.fromJson(jsonString);
        Dataset<Promotion> promotionDatasets = sparkSession.read().schema(structType).json(jsonPath).as(Encoders.bean(Promotion.class));
        promotionDatasets.printSchema();
        promotionDatasets.show();
        Dataset<Row> dataSetRowValue = sparkSession.read().schema(schema).json(jsonPath);
        dataSetRowValue.printSchema();
        dataSetRowValue.show();
        List<Row> rowList = dataSetRowValue.filter(dataSetRowValue.col("discount").$eq$eq$eq(10))
                .collectAsList();
        rowList.forEach(Unchecked.consumer(row -> log.info(row.json())));
        String jsonDir = System.getProperty("/Users/nisum/Desktop/EPE/sparkstructtype/src/main/resources/promotions.json");
        Dataset<Promotion> promotionDataset = sparkSession
                .read()
                .schema(schema)
                .json(jsonDir)
                .as(Encoders.bean(Promotion.class));
        promotionDataset.show();
        promotionDataset.createTempView("promotions");
        Dataset<Row> dataset = sparkSession.sql("select * from promotions where discount>=10");
        dataset.show();
        dataReader(jsonPath, Promotion.class).collect()
                .forEach(Unchecked.consumer(promotion -> log.info(promotion.toString())));

    }
    public static <T> JavaRDD<T> dataReader(String filePath, Class<T> clazz) {
        return sparkSession.read().textFile(filePath).toJavaRDD().map(
                value -> mapper.readValue(value, clazz));
    }
}
